@@ -75,9 +75,7 @@ public:
     template<typename Source>
     Mat4(Source *a)
     {
-        register int i;
-
-        for (i = 0; i < 16; i++) {
+        for (int i = 0; i < 16; i++) {
             MyBase::mm[i] = a[i];
         }
     }
