@@ -290,9 +290,6 @@ struct ISGradientNormSqrd<HJWENO5_BIAS>
     static typename Accessor::ValueType
     result(const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType     ValueType;
-        typedef math::Vec3<ValueType>            Vec3Type;
-
         // SSE optimized
         const simd::Float4
             v1(grid.getValue(ijk.offsetBy(-2, 0, 0)) - grid.getValue(ijk.offsetBy(-3, 0, 0)),
@@ -324,9 +321,6 @@ struct ISGradientNormSqrd<HJWENO5_BIAS>
     static typename StencilT::ValueType
     result(const StencilT& s)
     {
-        typedef typename StencilT::ValueType      ValueType;
-        typedef math::Vec3<ValueType>            Vec3Type;
-
         // SSE optimized
         const simd::Float4
             v1(s.template getValue<-2, 0, 0>() - s.template getValue<-3, 0, 0>(),
@@ -539,61 +533,79 @@ struct ISCurl
 template<DDScheme DiffScheme2, DScheme DiffScheme1>
 struct ISMeanCurvature
 {
-    // random access version
+    /// @brief random access version
+    /// @return true if the gradient is none-zero, in which case the
+    /// mean curvature is computed as two parts: @c alpha is the numerator in
+    /// @f$\nabla \cdot (\nabla \phi / |\nabla \phi|)@f$, and @c beta is @f$|\nabla \phi|@f$.
     template<typename Accessor>
-    static void result(const Accessor& grid, const Coord& ijk,
+    static bool result(const Accessor& grid, const Coord& ijk,
                        typename Accessor::ValueType& alpha,
                        typename Accessor::ValueType& beta)
     {
         typedef typename Accessor::ValueType ValueType;
 
-        ValueType Dx = D1<DiffScheme1>::inX(grid, ijk);
-        ValueType Dy = D1<DiffScheme1>::inY(grid, ijk);
-        ValueType Dz = D1<DiffScheme1>::inZ(grid, ijk);
+        const ValueType Dx = D1<DiffScheme1>::inX(grid, ijk);
+        const ValueType Dy = D1<DiffScheme1>::inY(grid, ijk);
+        const ValueType Dz = D1<DiffScheme1>::inZ(grid, ijk);
 
-        ValueType Dx2 = Dx*Dx;
-        ValueType Dy2 = Dy*Dy;
-        ValueType Dz2 = Dz*Dz;
+        const ValueType Dx2 = Dx*Dx;
+        const ValueType Dy2 = Dy*Dy;
+        const ValueType Dz2 = Dz*Dz;
+        const ValueType normGrad = Dx2 + Dy2 + Dz2;
+        if (normGrad <= math::Tolerance<ValueType>::value()) {
+            alpha = beta = 0;
+            return false;
+        }
 
-        ValueType Dxx = D2<DiffScheme2>::inX(grid, ijk);
-        ValueType Dyy = D2<DiffScheme2>::inY(grid, ijk);
-        ValueType Dzz = D2<DiffScheme2>::inZ(grid, ijk);
+        const ValueType Dxx = D2<DiffScheme2>::inX(grid, ijk);
+        const ValueType Dyy = D2<DiffScheme2>::inY(grid, ijk);
+        const ValueType Dzz = D2<DiffScheme2>::inZ(grid, ijk);
 
-        ValueType Dxy = D2<DiffScheme2>::inXandY(grid, ijk);
-        ValueType Dyz = D2<DiffScheme2>::inYandZ(grid, ijk);
-        ValueType Dxz = D2<DiffScheme2>::inXandZ(grid, ijk);
+        const ValueType Dxy = D2<DiffScheme2>::inXandY(grid, ijk);
+        const ValueType Dyz = D2<DiffScheme2>::inYandZ(grid, ijk);
+        const ValueType Dxz = D2<DiffScheme2>::inXandZ(grid, ijk);
 
         // for return
         alpha = (Dx2*(Dyy+Dzz)+Dy2*(Dxx+Dzz)+Dz2*(Dxx+Dyy)-2*(Dx*(Dy*Dxy+Dz*Dxz)+Dy*Dz*Dyz));
-        beta  = ValueType(std::sqrt(double(Dx2 + Dy2 + Dz2))); // * 1/dx
+        beta  = ValueType(std::sqrt(double(normGrad))); // * 1/dx
+        return true;
     }
 
-    // stencil access version
+    /// @brief stencil access version
+    /// @return true if the gradient is none-zero, in which case the
+    /// mean curvature is computed as two parts: @c alpha is the numerator in
+    /// @f$\nabla \cdot (\nabla \phi / |\nabla \phi|)@f$, and @c beta is @f$|\nabla \phi|@f$.
     template<typename StencilT>
-    static void result(const StencilT& stencil,
+    static bool result(const StencilT& stencil,
                        typename StencilT::ValueType& alpha,
                        typename StencilT::ValueType& beta)
     {
         typedef typename StencilT::ValueType   ValueType;
-        ValueType Dx = D1<DiffScheme1>::inX(stencil);
-        ValueType Dy = D1<DiffScheme1>::inY(stencil);
-        ValueType Dz = D1<DiffScheme1>::inZ(stencil);
+        const ValueType Dx = D1<DiffScheme1>::inX(stencil);
+        const ValueType Dy = D1<DiffScheme1>::inY(stencil);
+        const ValueType Dz = D1<DiffScheme1>::inZ(stencil);
 
-        ValueType Dx2 = Dx*Dx;
-        ValueType Dy2 = Dy*Dy;
-        ValueType Dz2 = Dz*Dz;
+        const ValueType Dx2 = Dx*Dx;
+        const ValueType Dy2 = Dy*Dy;
+        const ValueType Dz2 = Dz*Dz;
+        const ValueType normGrad = Dx2 + Dy2 + Dz2;
+        if (normGrad <= math::Tolerance<ValueType>::value()) {
+            alpha = beta = 0;
+            return false;
+        }
 
-        ValueType Dxx = D2<DiffScheme2>::inX(stencil);
-        ValueType Dyy = D2<DiffScheme2>::inY(stencil);
-        ValueType Dzz = D2<DiffScheme2>::inZ(stencil);
+        const ValueType Dxx = D2<DiffScheme2>::inX(stencil);
+        const ValueType Dyy = D2<DiffScheme2>::inY(stencil);
+        const ValueType Dzz = D2<DiffScheme2>::inZ(stencil);
 
-        ValueType Dxy = D2<DiffScheme2>::inXandY(stencil);
-        ValueType Dyz = D2<DiffScheme2>::inYandZ(stencil);
-        ValueType Dxz = D2<DiffScheme2>::inXandZ(stencil);
+        const ValueType Dxy = D2<DiffScheme2>::inXandY(stencil);
+        const ValueType Dyz = D2<DiffScheme2>::inYandZ(stencil);
+        const ValueType Dxz = D2<DiffScheme2>::inXandZ(stencil);
 
         // for return
         alpha = (Dx2*(Dyy+Dzz)+Dy2*(Dxx+Dzz)+Dz2*(Dxx+Dyy)-2*(Dx*(Dy*Dxy+Dz*Dxz)+Dy*Dz*Dyz));
-        beta = ValueType(std::sqrt(double(Dx2 + Dy2 + Dz2))); // * 1/dx
+        beta = ValueType(std::sqrt(double(normGrad))); // * 1/dx
+        return true;
     }
 };
 
@@ -922,7 +934,6 @@ struct Divergence
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const MapType& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -939,7 +950,6 @@ struct Divergence
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const MapType& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -962,7 +972,6 @@ struct Divergence<TranslationMap, DiffScheme>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const TranslationMap&, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -974,7 +983,6 @@ struct Divergence<TranslationMap, DiffScheme>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const TranslationMap&, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -992,7 +1000,6 @@ struct Divergence<UniformScaleMap, DiffScheme>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const UniformScaleMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1006,7 +1013,6 @@ struct Divergence<UniformScaleMap, DiffScheme>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const UniformScaleMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1026,7 +1032,6 @@ struct Divergence<UniformScaleTranslateMap, DiffScheme>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const UniformScaleTranslateMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1040,7 +1045,6 @@ struct Divergence<UniformScaleTranslateMap, DiffScheme>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const UniformScaleTranslateMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1060,7 +1064,6 @@ struct Divergence<UniformScaleMap, CD_2ND>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const UniformScaleMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1073,7 +1076,6 @@ struct Divergence<UniformScaleMap, CD_2ND>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const UniformScaleMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1092,7 +1094,6 @@ struct Divergence<UniformScaleTranslateMap, CD_2ND>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const UniformScaleTranslateMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1106,7 +1107,6 @@ struct Divergence<UniformScaleTranslateMap, CD_2ND>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const UniformScaleTranslateMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1126,7 +1126,6 @@ struct Divergence<ScaleMap, DiffScheme>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const ScaleMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div = ValueType(
@@ -1140,7 +1139,6 @@ struct Divergence<ScaleMap, DiffScheme>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const ScaleMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1161,7 +1159,6 @@ struct Divergence<ScaleTranslateMap, DiffScheme>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const ScaleTranslateMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div = ValueType(
@@ -1175,7 +1172,6 @@ struct Divergence<ScaleTranslateMap, DiffScheme>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const ScaleTranslateMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div(0);
@@ -1196,7 +1192,6 @@ struct Divergence<ScaleMap, CD_2ND>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const ScaleMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div = ValueType(
@@ -1210,7 +1205,6 @@ struct Divergence<ScaleMap, CD_2ND>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const ScaleMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div = ValueType(
@@ -1230,7 +1224,6 @@ struct Divergence<ScaleTranslateMap, CD_2ND>
     template<typename Accessor> static typename Accessor::ValueType::value_type
     result(const ScaleTranslateMap& map, const Accessor& grid, const Coord& ijk)
     {
-        typedef typename Accessor::ValueType             Vec3Type;
         typedef typename Accessor::ValueType::value_type ValueType;
 
         ValueType div = ValueType(
@@ -1244,7 +1237,6 @@ struct Divergence<ScaleTranslateMap, CD_2ND>
     template<typename StencilT> static typename StencilT::ValueType::value_type
     result(const ScaleTranslateMap& map, const StencilT& stencil)
     {
-        typedef typename StencilT::ValueType             Vec3Type;
         typedef typename StencilT::ValueType::value_type ValueType;
 
         ValueType div = ValueType(
@@ -1737,18 +1729,33 @@ struct CPT_RANGE
 template<typename MapType, DDScheme DiffScheme2, DScheme DiffScheme1>
 struct MeanCurvature
 {
-    // random access version
+    /// @brief random access version
+    /// @return true if the gradient is none-zero, in which case the
+    /// mean curvature is computed as two parts: @c alpha is the numerator in
+    /// @f$\nabla \cdot (\nabla \phi / |\nabla \phi|)@f$, and @c beta is @f$|\nabla \phi|@f$.
     template<typename Accessor>
-    static void compute(const MapType& map, const Accessor& grid, const Coord& ijk,
-                            double& alpha, double& beta)
+    static bool compute(const MapType& map, const Accessor& grid, const Coord& ijk,
+                        double& alpha, double& beta)
     {
         typedef typename Accessor::ValueType ValueType;
-        typedef Vec3<ValueType>              Vec3Type;
 
-         // compute the gradient in index space
+         // compute the gradient in index and world space
          Vec3d d1_is(D1<DiffScheme1>::inX(grid, ijk),
                      D1<DiffScheme1>::inY(grid, ijk),
-                     D1<DiffScheme1>::inZ(grid, ijk) );
+                     D1<DiffScheme1>::inZ(grid, ijk)), d1_ws;
+         if (is_linear<MapType>::value) {//resolved at compiletime
+             d1_ws = map.applyIJT(d1_is);
+         } else {
+             d1_ws = map.applyIJT(d1_is, ijk.asVec3d());
+         }
+         const double Dx2 = d1_ws(0)*d1_ws(0);
+         const double Dy2 = d1_ws(1)*d1_ws(1);
+         const double Dz2 = d1_ws(2)*d1_ws(2);
+         const double normGrad = Dx2 + Dy2 + Dz2;
+         if (normGrad <= math::Tolerance<double>::value()) {
+             alpha = beta = 0;
+             return false;
+         }
 
          // all the second derivatives in index space
          ValueType iddx  = D2<DiffScheme2>::inX(grid, ijk);
@@ -1764,28 +1771,21 @@ struct MeanCurvature
                       iddxy, iddy,  iddyz,
                       iddxz, iddyz, iddz);
 
-         // convert to range space
-         Mat3d d2_rs;
-         Vec3d d1_rs;
-         if (is_linear<MapType>::value) {
-             d2_rs = map.applyIJC(d2_is);
-             d1_rs = map.applyIJT(d1_is);
+         // convert second derivatives to world space
+         Mat3d d2_ws;
+         if (is_linear<MapType>::value) {//resolved at compiletime
+             d2_ws = map.applyIJC(d2_is);
          } else {
-             d2_rs = map.applyIJC(d2_is, d1_is, ijk.asVec3d());
-             d1_rs = map.applyIJT(d1_is, ijk.asVec3d());
+             d2_ws = map.applyIJC(d2_is, d1_is, ijk.asVec3d());
          }
 
-         // assemble the mean curvature
-         double Dx2 = d1_rs(0)*d1_rs(0);
-         double Dy2 = d1_rs(1)*d1_rs(1);
-         double Dz2 = d1_rs(2)*d1_rs(2);
-
-         // for return
-         alpha = (Dx2*(d2_rs(1,1)+d2_rs(2,2))+Dy2*(d2_rs(0,0)+d2_rs(2,2))
-                  +Dz2*(d2_rs(0,0)+d2_rs(1,1))
-                  -2*(d1_rs(0)*(d1_rs(1)*d2_rs(0,1)+d1_rs(2)*d2_rs(0,2))
-                      +d1_rs(1)*d1_rs(2)*d2_rs(1,2)));
-         beta  = std::sqrt(Dx2 + Dy2 + Dz2); // * 1/dx
+         // assemble the nominator and denominator for mean curvature
+         alpha = (Dx2*(d2_ws(1,1)+d2_ws(2,2))+Dy2*(d2_ws(0,0)+d2_ws(2,2))
+                  +Dz2*(d2_ws(0,0)+d2_ws(1,1))
+                  -2*(d1_ws(0)*(d1_ws(1)*d2_ws(0,1)+d1_ws(2)*d2_ws(0,2))
+                      +d1_ws(1)*d1_ws(2)*d2_ws(1,2)));
+         beta = std::sqrt(normGrad); // * 1/dx
+         return true;
     }
 
     template<typename Accessor>
@@ -1794,9 +1794,8 @@ struct MeanCurvature
     {
         typedef typename Accessor::ValueType ValueType;
         double alpha, beta;
-        compute(map, grid, ijk, alpha, beta);
-
-        return ValueType(alpha/(2. *math::Pow3(beta)));
+        return compute(map, grid, ijk, alpha, beta) ?
+               ValueType(alpha/(2. *math::Pow3(beta))) : 0;
     }
 
     template<typename Accessor>
@@ -1805,22 +1804,37 @@ struct MeanCurvature
     {
         typedef typename Accessor::ValueType ValueType;
         double alpha, beta;
-        compute(map, grid, ijk, alpha, beta);
-
-        return ValueType(alpha/(2. *math::Pow2(beta)));
+        return compute(map, grid, ijk, alpha, beta) ?
+               ValueType(alpha/(2. *math::Pow2(beta))) : 0;
     }
 
-    // stencil access version
+    /// @brief stencil access version
+    /// @return true if the gradient is none-zero, in which case the
+    /// mean curvature is computed as two parts: @c alpha is the numerator in
+    /// @f$\nabla \cdot (\nabla \phi / |\nabla \phi|)@f$, and @c beta is @f$|\nabla \phi|@f$.
     template<typename StencilT>
-    static void compute(const MapType& map, const StencilT& stencil, double& alpha, double& beta)
+    static bool compute(const MapType& map, const StencilT& stencil,
+                        double& alpha, double& beta)
     {
         typedef typename StencilT::ValueType  ValueType;
-        typedef Vec3<ValueType>              Vec3Type;
 
-         // compute the gradient in index space
+         // compute the gradient in index and world space
          Vec3d d1_is(D1<DiffScheme1>::inX(stencil),
                      D1<DiffScheme1>::inY(stencil),
-                     D1<DiffScheme1>::inZ(stencil) );
+                     D1<DiffScheme1>::inZ(stencil) ), d1_ws;
+         if (is_linear<MapType>::value) {//resolved at compiletime
+             d1_ws = map.applyIJT(d1_is);
+         } else {
+             d1_ws = map.applyIJT(d1_is, stencil.getCenterCoord().asVec3d());
+         }
+         const double Dx2 = d1_ws(0)*d1_ws(0);
+         const double Dy2 = d1_ws(1)*d1_ws(1);
+         const double Dz2 = d1_ws(2)*d1_ws(2);
+         const double normGrad = Dx2 + Dy2 + Dz2;
+         if (normGrad <= math::Tolerance<double>::value()) {
+             alpha = beta = 0;
+             return false;
+         }
 
          // all the second derivatives in index space
          ValueType iddx  = D2<DiffScheme2>::inX(stencil);
@@ -1836,28 +1850,21 @@ struct MeanCurvature
                       iddxy, iddy,  iddyz,
                       iddxz, iddyz, iddz);
 
-         // convert to range space
-         Mat3d d2_rs;
-         Vec3d d1_rs;
-         if (is_linear<MapType>::value) {
-             d2_rs = map.applyIJC(d2_is);
-             d1_rs = map.applyIJT(d1_is);
+         // convert second derivatives to world space
+         Mat3d d2_ws;
+         if (is_linear<MapType>::value) {//resolved at compiletime
+             d2_ws = map.applyIJC(d2_is);
          } else {
-             d2_rs = map.applyIJC(d2_is, d1_is, stencil.getCenterCoord().asVec3d());
-             d1_rs = map.applyIJT(d1_is, stencil.getCenterCoord().asVec3d());
+             d2_ws = map.applyIJC(d2_is, d1_is, stencil.getCenterCoord().asVec3d());
          }
 
-         // assemble the mean curvature
-         double Dx2 = d1_rs(0)*d1_rs(0);
-         double Dy2 = d1_rs(1)*d1_rs(1);
-         double Dz2 = d1_rs(2)*d1_rs(2);
-
          // for return
-         alpha = (Dx2*(d2_rs(1,1)+d2_rs(2,2))+Dy2*(d2_rs(0,0)+d2_rs(2,2))
-                  +Dz2*(d2_rs(0,0)+d2_rs(1,1))
-                  -2*(d1_rs(0)*(d1_rs(1)*d2_rs(0,1)+d1_rs(2)*d2_rs(0,2))
-                      +d1_rs(1)*d1_rs(2)*d2_rs(1,2)));
-         beta  = std::sqrt(Dx2 + Dy2 + Dz2); // * 1/dx
+         alpha = (Dx2*(d2_ws(1,1)+d2_ws(2,2))+Dy2*(d2_ws(0,0)+d2_ws(2,2))
+                  +Dz2*(d2_ws(0,0)+d2_ws(1,1))
+                  -2*(d1_ws(0)*(d1_ws(1)*d2_ws(0,1)+d1_ws(2)*d2_ws(0,2))
+                      +d1_ws(1)*d1_ws(2)*d2_ws(1,2)));
+         beta  = std::sqrt(normGrad); // * 1/dx
+         return true;
     }
 
     template<typename StencilT>
@@ -1866,8 +1873,8 @@ struct MeanCurvature
     {
         typedef typename StencilT::ValueType ValueType;
         double alpha, beta;
-        compute(map, stencil, alpha, beta);
-        return ValueType(alpha/(2*math::Pow3(beta)));
+        return compute(map, stencil, alpha, beta) ?
+               ValueType(alpha/(2*math::Pow3(beta))) : 0;
     }
 
     template<typename StencilT>
@@ -1875,9 +1882,8 @@ struct MeanCurvature
     {
         typedef typename StencilT::ValueType ValueType;
         double alpha, beta;
-        compute(map, stencil, alpha, beta);
-
-        return ValueType(alpha/(2*math::Pow2(beta)));
+        return compute(map, stencil, alpha, beta) ?
+               ValueType(alpha/(2*math::Pow2(beta))) : 0;
     }
 };
 
@@ -1893,9 +1899,8 @@ struct MeanCurvature<TranslationMap, DiffScheme2, DiffScheme1>
         typedef typename Accessor::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta);
-
-        return ValueType(alpha /(2*math::Pow3(beta)));
+        return ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta) ?
+               ValueType(alpha /(2*math::Pow3(beta))) : 0;
     }
 
     template<typename Accessor>
@@ -1905,9 +1910,8 @@ struct MeanCurvature<TranslationMap, DiffScheme2, DiffScheme1>
         typedef typename Accessor::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta);
-
-        return ValueType(alpha/(2*math::Pow2(beta)));
+        return ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta) ?
+               ValueType(alpha/(2*math::Pow2(beta))) : 0;
     }
 
     // stencil access version
@@ -1917,9 +1921,8 @@ struct MeanCurvature<TranslationMap, DiffScheme2, DiffScheme1>
         typedef typename StencilT::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta);
-
-        return ValueType(alpha /(2*math::Pow3(beta)));
+        return ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta) ?
+               ValueType(alpha /(2*math::Pow3(beta))) : 0;
     }
 
     template<typename StencilT>
@@ -1928,9 +1931,8 @@ struct MeanCurvature<TranslationMap, DiffScheme2, DiffScheme1>
         typedef typename StencilT::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta);
-
-        return ValueType(alpha/(2*math::Pow2(beta)));
+        return ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta) ?
+               ValueType(alpha/(2*math::Pow2(beta))) : 0;
     }
 };
 
@@ -1946,10 +1948,11 @@ struct MeanCurvature<UniformScaleMap, DiffScheme2, DiffScheme1>
         typedef typename Accessor::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta);
-        ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
-
-        return ValueType(alpha*inv2dx/math::Pow3(beta));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta)) {
+            ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
+            return ValueType(alpha*inv2dx/math::Pow3(beta));
+        }
+        return 0;
     }
 
     template<typename Accessor>
@@ -1959,10 +1962,11 @@ struct MeanCurvature<UniformScaleMap, DiffScheme2, DiffScheme1>
         typedef typename Accessor::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta);
-        ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
-
-        return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta)) {
+            ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
+            return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        }
+        return 0;
     }
 
     // stencil access version
@@ -1972,10 +1976,11 @@ struct MeanCurvature<UniformScaleMap, DiffScheme2, DiffScheme1>
         typedef typename StencilT::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta);
-        ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
-
-        return ValueType(alpha*inv2dx/math::Pow3(beta));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta)) {
+            ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
+            return ValueType(alpha*inv2dx/math::Pow3(beta));
+        }
+        return 0;
     }
 
     template<typename StencilT>
@@ -1984,10 +1989,11 @@ struct MeanCurvature<UniformScaleMap, DiffScheme2, DiffScheme1>
         typedef typename StencilT::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta);
-        ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
-
-        return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta)) {
+            ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
+            return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        }
+        return 0;
     }
 };
 
@@ -2002,10 +2008,11 @@ struct MeanCurvature<UniformScaleTranslateMap, DiffScheme2, DiffScheme1>
         typedef typename Accessor::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta);
-        ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
-
-        return ValueType(alpha*inv2dx/math::Pow3(beta));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta)) {
+            ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
+            return ValueType(alpha*inv2dx/math::Pow3(beta));
+        }
+        return 0;
     }
 
     template<typename Accessor> static typename Accessor::ValueType
@@ -2014,10 +2021,11 @@ struct MeanCurvature<UniformScaleTranslateMap, DiffScheme2, DiffScheme1>
         typedef typename Accessor::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta);
-        ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
-
-        return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(grid, ijk, alpha, beta)) {
+            ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
+            return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        }
+        return 0;
     }
 
     // stencil access version
@@ -2027,10 +2035,11 @@ struct MeanCurvature<UniformScaleTranslateMap, DiffScheme2, DiffScheme1>
         typedef typename StencilT::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta);
-        ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
-
-        return ValueType(alpha*inv2dx/math::Pow3(beta));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta)) {
+            ValueType inv2dx = ValueType(map.getInvTwiceScale()[0]);
+            return ValueType(alpha*inv2dx/math::Pow3(beta));
+        }
+        return 0;
     }
 
     template<typename StencilT> static typename StencilT::ValueType
@@ -2039,10 +2048,11 @@ struct MeanCurvature<UniformScaleTranslateMap, DiffScheme2, DiffScheme1>
         typedef typename StencilT::ValueType ValueType;
 
         ValueType alpha, beta;
-        ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta);
-        ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
-
-        return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        if (ISMeanCurvature<DiffScheme2, DiffScheme1>::result(stencil, alpha, beta)) {
+            ValueType invdxdx = ValueType(map.getInvScaleSqr()[0]);
+            return ValueType(alpha*invdxdx/(2*math::Pow2(beta)));
+        }
+        return 0;
     }
 };
 
