@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -79,8 +79,6 @@ public:
     /// @brief Create and return a shared pointer to a new transform.
     static Transform::Ptr createLinearTransform(double voxelSize = 1.0);
     static Transform::Ptr createLinearTransform(const Mat4R&);
-    OPENVDB_DEPRECATED static Transform::Ptr createFrustumTransform(const CoordBBox&, double taper,
-        double depth, double voxelSize = 1.0);
     static Transform::Ptr createFrustumTransform(const BBoxd&, double taper,
         double depth, double voxelSize = 1.0);
     //@}
@@ -88,35 +86,33 @@ public:
     /// Return @c true if the transformation map is exclusively linear/affine.
     bool isLinear() const { return mMap->isLinear(); }
 
+    /// Return @c true if the transform is equivalent to an idenity.
+    bool isIdentity() const ;
     /// Return the transformation map's type-name
     Name mapType() const { return mMap->type(); }
 
 
-    /// @deprecated Use preRotate() instead.
-    OPENVDB_DEPRECATED void rotate(double radians, const Axis axis = X_AXIS);
-    /// @deprecated Use postTranslate() instead.
-    OPENVDB_DEPRECATED void translate(const Vec3d&);
-    /// @deprecated Use preScale() instead.
-    OPENVDB_DEPRECATED void scale(const Vec3d&);
-    /// @deprecated Use preScale() instead.
-    OPENVDB_DEPRECATED void scale(double);
-    /// @deprecated Use preShear() instead.
-    OPENVDB_DEPRECATED void shear(double shear, Axis axis0, Axis axis1);
-
     //@{
     /// @brief Update the linear (affine) map by prepending or
-    /// postfixing the appropriate operation
+    /// postfixing the appropriate operation.  In the case of 
+    /// a frustum, the pre-operations apply to the linear part 
+    /// of the transform and not the entire transform, while the
+    /// post-operations are allways applied last.
     void preRotate(double radians, const Axis axis = X_AXIS);
     void preTranslate(const Vec3d&);
     void preScale(const Vec3d&);
     void preScale(double);
     void preShear(double shear, Axis axis0, Axis axis1);
+    void preMult(const Mat4d&);
+    void preMult(const Mat3d&);
 
     void postRotate(double radians, const Axis axis = X_AXIS);
     void postTranslate(const Vec3d&);
     void postScale(const Vec3d&);
     void postScale(double);
     void postShear(double shear, Axis axis0, Axis axis1);
+    void postMult(const Mat4d&);
+    void postMult(const Mat3d&);
     //@}
 
     /// Return the size of a voxel using the linear component of the map.
@@ -125,14 +121,6 @@ public:
     /// @note Maps that have a nonlinear component (e.g., perspective and frustum maps)
     /// have position-dependent voxel sizes.
     Vec3d voxelSize(const Vec3d& xyz) const { return mMap->voxelSize(xyz); }
-    /// @brief Return the voxel dimensions of the linear component of the map.
-    /// @deprecated Use voxelSize() instead.
-    OPENVDB_DEPRECATED Vec3d voxelDimensions() const { return mMap->voxelDimensions(); }
-    /// @brief Return the voxel dimensions at position (x, y, z).
-    /// @note Maps that have a nonlinear component (e.g., perspective and frustum maps)
-    /// have position-dependent voxel dimensions.
-    /// @deprecated Use voxelSize() instead.
-    OPENVDB_DEPRECATED Vec3d voxelDimensions(const Vec3d& xyz) const { return mMap->voxelDimensions(xyz); }
 
     /// Return the voxel volume of the linear component of the map.
     double voxelVolume() const { return mMap->determinant(); }
@@ -296,13 +284,12 @@ processTypedMap(TransformType& transform, OpType& op)
     return true;
 }
 
-
 } // namespace math
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_MATH_TRANSFORM_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012 DreamWorks Animation LLC
+// Copyright (c) 2012-2013 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
