@@ -71,9 +71,9 @@ template<typename ScalarGridType> struct ScalarToVectorConverter {
 /// @brief Compute the Closest-Point Transform (CPT) from a distance field.
 /// @return a new vector-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a DoubleGrid, the output grid will be a Vec3DGrid)
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 /// @note The current implementation assumes all the input distance values
 /// are represented by leaf voxels and not tiles.  This is true for all
 /// narrow-band level sets, which this class was originally developed for.
@@ -86,7 +86,6 @@ template<typename GridType, typename MaskT, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 cpt(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
 
-    
 template<typename GridType> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 cpt(const GridType& grid, bool threaded = true)
@@ -94,7 +93,6 @@ cpt(const GridType& grid, bool threaded = true)
     return cpt<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
-    
 template<typename GridType, typename MaskT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 cpt(const GridType& grid, const MaskT& mask, bool threaded = true)
@@ -105,9 +103,9 @@ cpt(const GridType& grid, const MaskT& mask, bool threaded = true)
 
 /// @brief Compute the curl of the given vector-valued grid.
 /// @return a new vector-valued grid
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 curl(const GridType& grid, bool threaded, InterruptT* interrupt);
@@ -115,7 +113,7 @@ curl(const GridType& grid, bool threaded, InterruptT* interrupt);
 template<typename GridType, typename MaskT, typename InterruptT> inline
 typename GridType::Ptr
 curl(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
-    
+
 template<typename GridType> inline
 typename GridType::Ptr
 curl(const GridType& grid, bool threaded = true)
@@ -130,12 +128,13 @@ curl(const GridType& grid, const MaskT& mask, bool threaded = true)
     return curl<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
 }
 
+
 /// @brief Compute the divergence of the given vector-valued grid.
 /// @return a new scalar-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a Vec3DGrid, the output grid will be a DoubleGrid)
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 divergence(const GridType& grid, bool threaded, InterruptT* interrupt);
@@ -143,7 +142,7 @@ divergence(const GridType& grid, bool threaded, InterruptT* interrupt);
 template<typename GridType, typename MaskT, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 divergence(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
-    
+
 template<typename GridType> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 divergence(const GridType& grid, bool threaded = true)
@@ -158,12 +157,13 @@ divergence(const GridType& grid, const MaskT& mask, bool threaded = true)
     return divergence<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
 }
 
+
 /// @brief Compute the gradient of the given scalar grid.
 /// @return a new vector-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a DoubleGrid, the output grid will be a Vec3DGrid)
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 gradient(const GridType& grid, bool threaded, InterruptT* interrupt);
@@ -179,7 +179,6 @@ gradient(const GridType& grid, bool threaded = true)
     return gradient<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
-
 template<typename GridType, typename MaskT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 gradient(const GridType& grid, const MaskT& mask, bool threaded = true)
@@ -190,9 +189,9 @@ gradient(const GridType& grid, const MaskT& mask, bool threaded = true)
 
 /// @brief Compute the Laplacian of the given scalar grid.
 /// @return a new scalar grid
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 laplacian(const GridType& grid, bool threaded, InterruptT* interrupt);
@@ -214,12 +213,13 @@ laplacian(const GridType& grid, const MaskT mask, bool threaded = true)
 {
     return laplacian<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
 }
-    
+
+
 /// @brief Compute the mean curvature of the given grid.
 /// @return a new grid
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 meanCurvature(const GridType& grid, bool threaded, InterruptT* interrupt);
@@ -230,24 +230,25 @@ meanCurvature(const GridType& grid, const MaskT& mask, bool threaded, InterruptT
 
 template<typename GridType> inline
 typename GridType::Ptr
-meanCurvature(const GridType& grid, bool threaded = true)    
+meanCurvature(const GridType& grid, bool threaded = true)
 {
     return meanCurvature<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
 template<typename GridType, typename MaskT> inline
 typename GridType::Ptr
-meanCurvature(const GridType& grid, const MaskT& mask, bool threaded = true)    
+meanCurvature(const GridType& grid, const MaskT& mask, bool threaded = true)
 {
     return meanCurvature<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
 }
 
+
 /// @brief Compute the magnitudes of the vectors of the given vector-valued grid.
 /// @return a new scalar-valued grid with the same numerical precision as the input grid
 ///     (for example, if the input grid is a Vec3DGrid, the output grid will be a DoubleGrid)
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 magnitude(const GridType& grid, bool threaded, InterruptT* interrupt);
@@ -255,26 +256,27 @@ magnitude(const GridType& grid, bool threaded, InterruptT* interrupt);
 template<typename GridType, typename MaskT, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 magnitude(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt);
-    
+
 template<typename GridType> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
-magnitude(const GridType& grid, bool threaded = true) 
+magnitude(const GridType& grid, bool threaded = true)
 {
     return magnitude<GridType, util::NullInterrupter>(grid, threaded, NULL);
 }
 
 template<typename GridType, typename MaskT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
-magnitude(const GridType& grid, const MaskT& mask, bool threaded = true) 
+magnitude(const GridType& grid, const MaskT& mask, bool threaded = true)
 {
     return magnitude<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
 }
-    
+
+
 /// @brief Normalize the vectors of the given vector-valued grid.
 /// @return a new vector-valued grid
-/// When a mask grid is specified, the solution is calculated only in the intersection of
-/// the mask active topology and the input active topology independent of the 
-/// transforms associated with either grid.
+/// @details When a mask grid is specified, the solution is calculated only in
+/// the intersection of the mask active topology and the input active topology
+/// independent of the transforms associated with either grid.
 template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 normalize(const GridType& grid, bool threaded, InterruptT* interrupt);
@@ -296,11 +298,21 @@ normalize(const GridType& grid, const MaskT& mask, bool threaded = true)
 {
     return normalize<GridType, MaskT, util::NullInterrupter>(grid, mask, threaded, NULL);
 }
-    
+
+
 ////////////////////////////////////////
 
 
-namespace {
+namespace gridop {
+
+/// @brief ToBoolGrid<T>::Type is the type of a grid having the same
+/// tree hierarchy as grid type T but a value type of bool.
+/// @details For example, ToBoolGrid<FloatGrid>::Type is equivalent to BoolGrid.
+template<typename GridType>
+struct ToBoolGrid {
+    typedef Grid<typename GridType::TreeType::template ValueConverter<bool>::Type> Type;
+};
+
 
 /// @brief Apply an operator on an input grid to produce an output grid
 /// with the same topology but a possibly different value type.
@@ -311,19 +323,26 @@ namespace {
 /// @note The current implementation assumes all the input
 /// values are represented by leaf voxels and not tiles. In the
 /// future we will expand this class to also handle tile values.
-template<typename InGridT, typename MaskGridType, typename OutGridT, typename MapT, typename OperatorT, typename InterruptT = util::NullInterrupter>
+template<
+    typename InGridT,
+    typename MaskGridType,
+    typename OutGridT,
+    typename MapT,
+    typename OperatorT,
+    typename InterruptT = util::NullInterrupter>
 class GridOperator
 {
 public:
     typedef typename OutGridT::TreeType           OutTreeT;
     typedef typename OutTreeT::LeafNodeType       OutLeafT;
     typedef typename tree::LeafManager<OutTreeT>  LeafManagerT;
-  
-    GridOperator(const InGridT& grid, const MaskGridType* mask, const MapT& map, InterruptT* interrupt = NULL):
+
+    GridOperator(const InGridT& grid, const MaskGridType* mask, const MapT& map,
+        InterruptT* interrupt = NULL):
         mAcc(grid.getConstAccessor()), mMap(map), mInterrupt(interrupt), mMask(mask)
     {
     }
-  
+
     virtual ~GridOperator() {}
     typename OutGridT::Ptr process(bool threaded = true)
     {
@@ -335,27 +354,27 @@ public:
 
         // output tree = topology copy of input tree!
         typename OutTreeT::Ptr tree(new OutTreeT(mAcc.tree(), backg, TopologyCopy()));
-        
-        
+
+
         // create grid with output tree and unit transform
         typename OutGridT::Ptr result(new OutGridT(tree));
-        
+
         // Modify the solution area if a mask was supplied.
         if (mMask) {
-            result->topologyIntersection(*mMask); 
+            result->topologyIntersection(*mMask);
         }
 
         // transform of output grid = transform of input grid
         result->setTransform(math::Transform::Ptr(new math::Transform( mMap.copy() )));
-       
+
         LeafManagerT leafManager(*tree);
-        
+
         if (threaded) {
             tbb::parallel_for(leafManager.leafRange(), *this);
         } else {
             (*this)(leafManager.leafRange());
         }
-        
+
         if (mInterrupt) mInterrupt->end();
         return result;
     }
@@ -369,7 +388,7 @@ public:
     void operator()(const typename LeafManagerT::LeafRange& range) const
     {
         if (util::wasInterrupted(mInterrupt)) tbb::task::self().cancel_group_execution();
-        
+
         for (typename LeafManagerT::LeafRange::Iterator leaf=range.begin(); leaf; ++leaf) {
             for (typename OutLeafT::ValueOnIter value=leaf->beginValueOn(); value; ++value) {
                 value.setValue(OperatorT::result(mMap, mAcc, value.getCoord()));
@@ -378,7 +397,6 @@ public:
     }
 
 protected:
-    
     typedef typename InGridT::ConstAccessor  AccessorT;
     mutable AccessorT   mAcc;
     const MapT&         mMap;
@@ -386,14 +404,17 @@ protected:
     const MaskGridType* mMask;
 }; // end of GridOperator class
 
-} //end of anonymous namespaceo
+} // namespace gridop
 
 
 ////////////////////////////////////////
 
 
 /// @brief Compute the closest-point transform of a scalar grid.
-template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+template<
+    typename InGridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<InGridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class Cpt
 {
 public:
@@ -405,19 +426,19 @@ public:
     {
     }
 
-
     Cpt(const InGridType& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
 
-
     typename OutGridType::Ptr process(bool threaded = true, bool useWorldTransform = true)
     {
         Functor functor(mInputGrid, mMask, threaded, useWorldTransform, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
+        if (functor.mOutputGrid) functor.mOutputGrid->setVectorType(VEC_CONTRAVARIANT_ABSOLUTE);
         return functor.mOutputGrid;
     }
+
 private:
     struct IsOpT
     {
@@ -439,16 +460,25 @@ private:
     };
     struct Functor
     {
-        Functor(const InGridType& grid, const MaskGridType* mask, bool threaded, bool worldspace, InterruptT* interrupt):
-            mThreaded(threaded), mWorldSpace(worldspace), mInputGrid(grid), mInterrupt(interrupt), mMask(mask){}
+        Functor(const InGridType& grid, const MaskGridType* mask,
+            bool threaded, bool worldspace, InterruptT* interrupt)
+        : mThreaded(threaded)
+        , mWorldSpace(worldspace)
+        , mInputGrid(grid)
+        , mInterrupt(interrupt)
+        , mMask(mask)
+        {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
             if (mWorldSpace) {
-                GridOperator<InGridType, MaskGridType, OutGridType, MapT, WsOpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
+                gridop::GridOperator<InGridType, MaskGridType, OutGridType, MapT, WsOpT, InterruptT>
+                    op(mInputGrid, mMask, map, mInterrupt);
                 mOutputGrid = op.process(mThreaded); // cache the result
             } else {
-                GridOperator<InGridType, MaskGridType, OutGridType, MapT, IsOpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
+                gridop::GridOperator<InGridType, MaskGridType, OutGridType, MapT, IsOpT, InterruptT>
+                    op(mInputGrid, mMask, map, mInterrupt);
                 mOutputGrid = op.process(mThreaded); // cache the result
             }
         }
@@ -468,17 +498,22 @@ private:
 ////////////////////////////////////////
 
 
-/// @brief Compute the curl of a scalar grid.
-template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+/// @brief Compute the curl of a vector grid.
+template<
+    typename GridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<GridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class Curl
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
+
     Curl(const GridT& grid, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+
     Curl(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
@@ -488,22 +523,26 @@ public:
     {
         Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
+        if (functor.mOutputGrid) functor.mOutputGrid->setVectorType(VEC_COVARIANT);
         return functor.mOutputGrid;
     }
 
 private:
     struct Functor
     {
-        Functor(const GridT& grid, const MaskGridType* mask, 
+        Functor(const GridT& grid, const MaskGridType* mask,
                 bool threaded, InterruptT* interrupt):
-            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask){}
+            mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Curl<MapT, math::CD_2ND> OpT;
-            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
+            gridop::GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT>
+                op(mInputGrid, mMask, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
+
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
@@ -520,8 +559,11 @@ private:
 ////////////////////////////////////////
 
 
-/// @brief Computes the Divergence of a scalar grid
-template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+/// @brief Compute the divergence of a vector grid.
+template<
+    typename InGridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<InGridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class Divergence
 {
 public:
@@ -537,14 +579,14 @@ public:
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
+
     typename OutGridType::Ptr process(bool threaded = true)
     {
-        if( mInputGrid.getGridClass() == GRID_STAGGERED ) {
+        if (mInputGrid.getGridClass() == GRID_STAGGERED) {
             Functor<math::FD_1ST> functor(mInputGrid, mMask, threaded, mInterrupt);
             processTypedMap(mInputGrid.transform(), functor);
             return functor.mOutputGrid;
-        }
-        else {
+        } else {
             Functor<math::CD_2ND> functor(mInputGrid, mMask, threaded, mInterrupt);
             processTypedMap(mInputGrid.transform(), functor);
             return functor.mOutputGrid;
@@ -555,15 +597,19 @@ protected:
     template<math::DScheme DiffScheme>
     struct Functor
     {
-        Functor(const InGridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+        Functor(const InGridT& grid, const MaskGridType* mask,
+            bool threaded, InterruptT* interrupt):
             mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Divergence<MapT, DiffScheme> OpT;
-            GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
+            gridop::GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT>
+                op(mInputGrid, mMask, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
+
         const bool                 mThreaded;
         const InGridType&          mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
@@ -580,8 +626,11 @@ protected:
 ////////////////////////////////////////
 
 
-/// @brief Computes the Gradient of a scalar grid
-template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+/// @brief Compute the gradient of a scalar grid.
+template<
+    typename InGridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<InGridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class Gradient
 {
 public:
@@ -592,29 +641,36 @@ public:
         mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+
     Gradient(const InGridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
+
     typename OutGridType::Ptr process(bool threaded = true)
     {
         Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
+        if (functor.mOutputGrid) functor.mOutputGrid->setVectorType(VEC_COVARIANT);
         return functor.mOutputGrid;
     }
 
 protected:
     struct Functor
     {
-        Functor(const InGridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+        Functor(const InGridT& grid, const MaskGridType* mask,
+            bool threaded, InterruptT* interrupt):
             mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Gradient<MapT, math::CD_2ND> OpT;
-            GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, mMask, map, mInterrupt);
+            gridop::GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT>
+                op(mInputGrid, mMask, map, mInterrupt);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
+
         const bool                 mThreaded;
         const InGridT&             mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
@@ -631,26 +687,31 @@ protected:
 ////////////////////////////////////////
 
 
-/// @brief Computes the Laplacian of a scalar grid
-template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+template<
+    typename GridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<GridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class Laplacian
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
+
     Laplacian(const GridT& grid, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
-    
+
     Laplacian(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
+
     typename GridT::Ptr process(bool threaded = true)
     {
         Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
+        if (functor.mOutputGrid) functor.mOutputGrid->setVectorType(VEC_COVARIANT);
         return functor.mOutputGrid;
     }
 
@@ -659,13 +720,16 @@ protected:
     {
         Functor(const GridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
             mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::Laplacian<MapT, math::CD_SECOND> OpT;
-            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask, map);
+            gridop::GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT>
+                op(mInputGrid, mMask, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
+
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
@@ -682,16 +746,21 @@ protected:
 ////////////////////////////////////////
 
 
-template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+template<
+    typename GridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<GridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class MeanCurvature
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
+
     MeanCurvature(const GridT& grid, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+
     MeanCurvature(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
@@ -701,6 +770,7 @@ public:
     {
         Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
+        if (functor.mOutputGrid) functor.mOutputGrid->setVectorType(VEC_COVARIANT);
         return functor.mOutputGrid;
     }
 
@@ -709,18 +779,21 @@ protected:
     {
         Functor(const GridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
             mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
             typedef math::MeanCurvature<MapT, math::CD_SECOND, math::CD_2ND> OpT;
-            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask, map);
+            gridop::GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT>
+                op(mInputGrid, mMask, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
+
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
         InterruptT*          mInterrupt;
-        const MaskGridType*  mMask; 
+        const MaskGridType*  mMask;
     }; // Private Functor
 
     const GridT&        mInputGrid;
@@ -732,20 +805,26 @@ protected:
 ////////////////////////////////////////
 
 
-template<typename InGridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+template<
+    typename InGridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<InGridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class Magnitude
 {
 public:
     typedef InGridT                                         InGridType;
     typedef typename VectorToScalarConverter<InGridT>::Type OutGridType;
+
     Magnitude(const InGridType& grid, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+
     Magnitude(const InGridType& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
+
     typename OutGridType::Ptr process(bool threaded = true)
     {
         Functor functor(mInputGrid, mMask, threaded, mInterrupt);
@@ -762,14 +841,18 @@ protected:
     };
     struct Functor
     {
-        Functor(const InGridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
+        Functor(const InGridT& grid, const MaskGridType* mask,
+            bool threaded, InterruptT* interrupt):
             mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
-            GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT> op(mInputGrid, mMask, map);
+            gridop::GridOperator<InGridType, MaskGridType, OutGridType, MapT, OpT, InterruptT>
+                op(mInputGrid, mMask, map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
+
         const bool                 mThreaded;
         const InGridType&          mInputGrid;
         typename OutGridType::Ptr  mOutputGrid;
@@ -786,25 +869,38 @@ protected:
 ////////////////////////////////////////
 
 
-template<typename GridT, typename MaskGridType = BoolGrid, typename InterruptT = util::NullInterrupter>
+template<
+    typename GridT,
+    typename MaskGridType = typename gridop::ToBoolGrid<GridT>::Type,
+    typename InterruptT = util::NullInterrupter>
 class Normalize
 {
 public:
     typedef GridT  InGridType;
     typedef GridT  OutGridType;
+
     Normalize(const GridT& grid, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(NULL)
     {
     }
+
     Normalize(const GridT& grid, const MaskGridType& mask, InterruptT* interrupt = NULL):
         mInputGrid(grid), mInterrupt(interrupt), mMask(&mask)
     {
     }
-  
+
     typename GridT::Ptr process(bool threaded = true)
     {
         Functor functor(mInputGrid, mMask, threaded, mInterrupt);
         processTypedMap(mInputGrid.transform(), functor);
+        if (typename GridT::Ptr outGrid = functor.mOutputGrid) {
+            const VecType vecType = mInputGrid.getVectorType();
+            if (vecType == VEC_COVARIANT) {
+                outGrid->setVectorType(VEC_COVARIANT_NORMALIZE);
+            } else {
+                outGrid->setVectorType(vecType);
+            }
+        }
         return functor.mOutputGrid;
     }
 
@@ -824,12 +920,15 @@ protected:
     {
         Functor(const GridT& grid, const MaskGridType* mask, bool threaded, InterruptT* interrupt):
             mThreaded(threaded), mInputGrid(grid), mInterrupt(interrupt), mMask(mask) {}
+
         template<typename MapT>
         void operator()(const MapT& map)
         {
-            GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT> op(mInputGrid, mMask,map);
+            gridop::GridOperator<GridT, MaskGridType, GridT, MapT, OpT, InterruptT>
+                op(mInputGrid, mMask,map);
             mOutputGrid = op.process(mThreaded); // cache the result
         }
+
         const bool           mThreaded;
         const GridT&         mInputGrid;
         typename GridT::Ptr  mOutputGrid;
@@ -850,7 +949,7 @@ template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 cpt(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Cpt<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    Cpt<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT> op(grid, interrupt);
     return op.process(threaded);
 }
 
@@ -866,11 +965,10 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 curl(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Curl<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    Curl<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT> op(grid, interrupt);
     return op.process(threaded);
 }
 
-
 template<typename GridType, typename MaskT, typename InterruptT> inline
 typename GridType::Ptr
 curl(const GridType& grid, const MaskT& mask, bool threaded, InterruptT* interrupt)
@@ -883,7 +981,8 @@ template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 divergence(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Divergence<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    Divergence<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT>
+        op(grid, interrupt);
     return op.process(threaded);
 }
 
@@ -899,7 +998,8 @@ template<typename GridType, typename InterruptT> inline
 typename ScalarToVectorConverter<GridType>::Type::Ptr
 gradient(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Gradient<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    Gradient<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT>
+        op(grid, interrupt);
     return op.process(threaded);
 }
 
@@ -915,7 +1015,8 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 laplacian(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Laplacian<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    Laplacian<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT>
+        op(grid, interrupt);
     return op.process(threaded);
 }
 
@@ -931,7 +1032,8 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 meanCurvature(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    MeanCurvature<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    MeanCurvature<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT>
+        op(grid, interrupt);
     return op.process(threaded);
 }
 
@@ -947,7 +1049,8 @@ template<typename GridType, typename InterruptT> inline
 typename VectorToScalarConverter<GridType>::Type::Ptr
 magnitude(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Magnitude<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    Magnitude<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT>
+        op(grid, interrupt);
     return op.process(threaded);
 }
 
@@ -963,7 +1066,8 @@ template<typename GridType, typename InterruptT> inline
 typename GridType::Ptr
 normalize(const GridType& grid, bool threaded, InterruptT* interrupt)
 {
-    Normalize<GridType, BoolGrid, InterruptT> op(grid, interrupt);
+    Normalize<GridType, typename gridop::ToBoolGrid<GridType>::Type, InterruptT>
+        op(grid, interrupt);
     return op.process(threaded);
 }
 
