@@ -202,7 +202,7 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& frus
         calculateBounds<math::NonlinearFrustumMap>(frustum, bounding_box, out);
         return;
     }
-    
+
     // for convenience
     Vec3d& out_min = out.min();
     Vec3d& out_max = out.max();
@@ -251,8 +251,6 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& frus
         // convert back to voxel space
         pre_image = frustum.applyInverseMap(extreme2);
         out_max.x() = std::max(x_min, std::min(x_max, pre_image.x()));
-        
-        const Vec3d tmpPlus = extreme2;
 
         extreme.x() = xm;
         extreme.y() = centerLS.y();
@@ -260,8 +258,6 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& frus
         // location in world space of the tangent point
         extreme2 = secondMap.applyMap(extreme);
 
-        const Vec3d tmpMinus = extreme2;
-
         // convert back to voxel space
         pre_image = frustum.applyInverseMap(extreme2);
         out_min.x() = std::max(x_min, std::min(x_max, pre_image.x()));
@@ -285,15 +281,11 @@ calculateBounds<math::NonlinearFrustumMap>(const math::NonlinearFrustumMap& frus
         pre_image = frustum.applyInverseMap(extreme2);
         out_max.y() = std::max(y_min, std::min(y_max, pre_image.y()));
 
-        const Vec3d tmpPlus = extreme2;
-
         extreme.x() = centerLS.x();
         extreme.y() = xm;
         extreme.z() = zm;
         extreme2 = secondMap.applyMap(extreme);
 
-        const Vec3d tmpMinus = extreme2;
-
         // convert back to voxel space
         pre_image = frustum.applyInverseMap(extreme2);
         out_min.y() = std::max(y_min, std::min(y_max, pre_image.y()));
